test_merchant_uuid = "ded91cc8-9e8d-44f6-925d-7188dac90de4"
test_api_token_payment = "RXCBB3ir4xLe8tmzQySBIeyJmkEd7SjbZpFPcm2s6fXigdEejNPKZBIPK6amjxCKFLrqduPjnaqZbneSD0A8idfLibMGkjTUyIcEYnMKEmqB5poW4cdm6nFKWZzKXrP8"
test_api_token_payout = "GzFfltvoDP5Vb4BQjTsUb6BTr9uJRDFIaScOy8W6zlAQTpDOYzcxck1DcXca2NPFvqae4Wfip0KyMUp2DkCLBKjc0HS2C88mtTwxikeADpKpYVMvHeyG63tdgeLChO8h"
